#!/bin/bash

### input:  peptide_intensity_gene.txt
###         tab delimited file with three columns: peptide sequence, intensity, gene name
###         peptides might have multiple entries
###         if multiple genes are assigned to a sequence, please coma separate the names
### output: text file with lists of genes and assigned intensities
###         if you would like to retain temporary files with peptide sequences and weights assigned during convergence, please comment the last line in the while loop
###
### based on "The EM Algorithm and Extensions", by Geoffrey McLachlan, Thriyambakam Krishnan, John Wiley & Sons, Nov 9, 2007
### inspired by Heiko Horn
### contact edytam@broadinstitute.org in case of any questions

counter=1

### get list of intensities per unique peptide; initiate with weights 1; calculate new weight per peptide
awk '{map[toupper($1)" "$3] += $2;} END {for (i in map) {print i, map[i] | "sort -t't'" }}' peptide_intensity_gene.txt | awk '{if($2 !~ /,/) print $0, $3; else {n=split($2,a,","); for (i in a) print $1, a[i], $3, $3*1/n;}}' > peptide_$counter
### calculate new weight per gene
awk '{map[$2] += $4;} END {for (i in map) {print i, map[i] | "sort -t't'" }}' peptide_$counter > gene_$counter
### replace old weight per peptide by new one per gene
awk 'FNR==NR{map[$1]=$2; next};{print $1, $2, $3, map[$2]}' gene_$counter peptide_$counter > tmp_$counter
cp tmp_$counter peptide_$counter

max_diff=1000000
echo $max_diff

while (( $max_diff > 1 ))
do
	old=$counter
	new=$(( counter+1 ))
	### calculate new weight per peptide
	awk 'FNR==NR{map[$1] += $4; next};{if (map[$1]>0) print $1, $2, $3, $3*$4/map[$1]; else print $1, $2, $3, "0"}' tmp_$old tmp_$old > peptide_$new
	### calculate new weight per gene
	awk '{map[$2] += $4;} END {for (i in map) if (map[i]<500) map[i]=0; for (i in map) {print i, map[i] | "sort -t't'" }}' peptide_$new > gene_$new
	### replace old weight per peptide by new one per gene
	awk 'FNR==NR{map[$1]=$2; next};{print $1, $2, $3, map[$2]}' gene_$new tmp_$old > tmp_$new
	### calculate difference between intensities assigned in steps old and new
	max_diff=$(paste gene_$old gene_$new | awk -v "max=0" '{if(($4-$2)>max) max=($4-$2)}END{printf("%d"), max}')
	echo $counter $max_diff
	(( counter++ ))
	rm tmp_$old peptide_$old
done

